/* qcam-FreeBSD.c -- FreeBSD-specific routines for accessing QuickCam */


#include <stdio.h>
#include <unistd.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <machine/cpufunc.h> /* inb & outb */

#include "qcam.h"

int read_lpstatus(struct qcam *q) { return inb(q->port+1); }
int read_lpcontrol(struct qcam *q) { return inb(q->port+2); }
int read_lpdata(struct qcam *q) { return inb(q->port); }
void write_lpdata(struct qcam *q, int d) { outb(q->port,d); }
void write_lpcontrol(struct qcam *q, int d) { outb(q->port+2,d); }

FILE *IO = NULL;

int enable_ports(struct qcam *q) 
{
  if(q->port<0x278) return 1; /* Better safe than sorry */
  if(q->port>0x3bc) return 1; 
  if (IO!=NULL)
    return 0; /* Already got it */
  IO=fopen("/dev/io","r+"); /* Simply opening /dev/io allows I/O instructions */
  return (IO==NULL);
}

int disable_ports(struct qcam *q)
{
  if (IO==NULL)
    return 0;
  fclose(IO);
  IO=NULL;
  return 0;
}

/* Lock port.  Not supported. */

int qc_lock_wait(struct qcam *q, int wait)
{
  return 0;
}

int qc_lock(struct qcam *q)
{
  return 0;
}

/* Unlock port */

int qc_unlock(struct qcam *q)
{
  return 0;
}

/* Probe for camera.  Returns 0 if found, 1 if not found, sets
   q->port.*/

int qc_probe(struct qcam *q)
{
  int ioports[]={0x378, 0x278, 0x3bc,0};
  int i=0;

  /* Attempt to get permission to access IO ports.  Must be root */

  while(ioports[i]!=0) {
    q->port=ioports[i++];

    if (qc_open(q)) {
      perror("Can't get I/O permission");
      exit(1);
    }

    if(qc_detect(q)) {
      fprintf(stderr,"QuickCam detected at 0x%x\n",q->port);
      qc_close(q);
      return(0);
    }
    else
    qc_close(q);
  }

  return 1;
}
